{% extends "base.html" %}

{% block title %}Hotkeys{% endblock %}
{% block header %}Hotkeys{% endblock %}

{% block body %}

<blockquote>
<p>Note: Before using the keyboard you must set the focus within the frame by clicking once on the frame.</p>
</blockquote>
<p>Some of these actions refer to the &quot;current word&quot; - that's either the word you're hovering over, or have clicked.</p>
<h2 id="navigation"><a class="header" href="#navigation">Navigation</a></h2>
<div class="table-wrapper"><table><thead><tr><th>Key</th><th>Action</th></tr></thead><tbody>
<tr><td><code>Right</code></td><td>Move to next word</td></tr>
<tr><td><code>Left</code></td><td>Move to previous word</td></tr>
<tr><td><code>Esc</code></td><td>Reset marked term(s)</td></tr>
</tbody></table>
</div>
<h2 id="misc-actions"><a class="header" href="#misc-actions">Misc actions</a></h2>
<div class="table-wrapper"><table><thead><tr><th>Key</th><th>Action</th></tr></thead><tbody>
<tr><td><code>t</code></td><td>Translate the sentence of the current word</td></tr>
<tr><td><code>Shift</code> + <code>t</code></td><td>Translate the paragraph of the current word</td></tr>
<tr><td><code>c</code></td><td>Copy the sentence of the current word to the clipboard</td></tr>
<tr><td><code>Shift</code> + <code>c</code></td><td>Copy the paragraph of the current word to the clipboard</td></tr>
<tr><td><code>m</code></td><td>Change to the next theme</td></tr>
<tr><td><code>h</code></td><td>Toggle highlights</td></tr>
<tr><td><code>f</code></td><td>Toggle focus mode</td></tr>
</tbody></table>
</div>
<h2 id="updating-status"><a class="header" href="#updating-status">Updating status</a></h2>
<p>You can edit the status of the current word with a keystroke:</p>
<div class="table-wrapper"><table><thead><tr><th>Key</th><th>Set status of marked term(s) to</th></tr></thead><tbody>
<tr><td><code>1</code> - <code>5</code></td><td>1 (Learning) to 5 (Learned)</td></tr>
<tr><td><code>i</code></td><td>&quot;Ignored&quot;</td></tr>
<tr><td><code>w</code></td><td>&quot;Well Known&quot;</td></tr>
</tbody></table>
</div>
<p>You can also &quot;bulk update&quot; statuses by holding down Shift and clicking with the mouse.  All of the underlined terms will then be affected when you hit 1, 2, etc.</p>


{% endblock %}
